@@ -2,29 +2,19 @@ from __future__ import annotations
 
 import inspect
 from abc import ABC, abstractmethod
-from collections import ChainMap, OrderedDict
-from contextlib import ContextDecorator, contextmanager
+from collections import OrderedDict
+from collections.abc import Callable, Iterable, Iterator, Mapping
+from contextlib import ContextDecorator, contextmanager, suppress
 from dataclasses import dataclass, field
 from enum import Enum, auto
 from functools import cached_property, singledispatchmethod, wraps
 from inspect import Signature, get_annotations
 from logging import getLogger
 from types import MappingProxyType
-from typing import (
-    Any,
-    Callable,
-    Iterable,
-    Iterator,
-    Mapping,
-    NamedTuple,
-    Protocol,
-    TypeVar,
-    cast,
-    final,
-    runtime_checkable,
-)
+from typing import Any, NamedTuple, Protocol, TypeVar, cast, final, runtime_checkable
 
 from injection.common.event import Event, EventChannel, EventListener
+from injection.common.formatting import format_type
 from injection.common.lazy import Lazy, LazyMapping
 from injection.exceptions import (
     ModuleCircularUseError,
@@ -40,13 +30,6 @@ _logger = getLogger(__name__)
 T = TypeVar("T")
 
 
-def _format_type(cls: type) -> str:
-    try:
-        return f"{cls.__module__}.{cls.__qualname__}"
-    except AttributeError:
-        return str(cls)
-
-
 """
 Events
 """
@@ -59,16 +42,14 @@ class ContainerEvent(Event, ABC):
 
 @dataclass(frozen=True, slots=True)
 class ContainerDependenciesUpdated(ContainerEvent):
-    references: set[type]
+    classes: set[type]
 
     def __str__(self) -> str:
-        length = len(self.references)
-        formatted_references = ", ".join(
-            f"`{_format_type(reference)}`" for reference in self.references
-        )
+        length = len(self.classes)
+        formatted_classes = ", ".join(f"`{format_type(cls)}`" for cls in self.classes)
         return (
             f"{length} container dependenc{'ies' if length > 1 else 'y'} have been "
-            f"updated{f': {formatted_references}' if formatted_references else ''}."
+            f"updated{f': {formatted_classes}' if formatted_classes else ''}."
         )
 
 
@@ -84,29 +65,37 @@ class ModuleEventProxy(ModuleEvent):
     def __str__(self) -> str:
         return f"`{self.on_module}` has propagated an event: {self.origin}"
 
-    def __iter__(self) -> Iterator[Event]:
+    @property
+    def history(self) -> Iterator[Event]:
         if isinstance(self.event, ModuleEventProxy):
-            yield from self.event
+            yield from self.event.history
 
         yield self.event
 
     @property
     def origin(self) -> Event:
-        return next(iter(self))
+        return next(self.history)
 
-    @property
-    def is_circular(self) -> bool:
-        return any(
-            isinstance(event, ModuleEvent) and event.on_module is self.on_module
-            for event in self
-        )
+    def check_recursion(self):
+        last_module = None
+        found = False
 
-    @property
-    def previous_module(self) -> Module | None:
-        if isinstance(self.event, ModuleEvent):
-            return self.event.on_module
+        for event in self.history:
+            if not isinstance(event, ModuleEvent):
+                continue
 
-        return None
+            last_module = event.on_module
+
+            if not found:
+                found = last_module is self.on_module
+
+        if found:
+            raise ModuleCircularUseError(
+                "Circular dependency between two modules: "
+                f"`{self.on_module}` and `{last_module}`."
+            )
+
+        return self
 
 
 @dataclass(frozen=True, slots=True)
@@ -152,18 +141,18 @@ class Injectable(Protocol[T]):
 
 
 @dataclass(repr=False, frozen=True, slots=True)
-class _BaseInjectable(Injectable[T], ABC):
+class BaseInjectable(Injectable[T], ABC):
     factory: Callable[[], T]
 
 
-class NewInjectable(_BaseInjectable[T]):
+class NewInjectable(BaseInjectable[T]):
     __slots__ = ()
 
     def get_instance(self) -> T:
         return self.factory()
 
 
-class SingletonInjectable(_BaseInjectable[T]):
+class SingletonInjectable(BaseInjectable[T]):
     @cached_property
     def __instance(self) -> T:
         return self.factory()
@@ -182,36 +171,32 @@ class Container:
     __data: dict[type, Injectable] = field(default_factory=dict, init=False)
     __channel: EventChannel = field(default_factory=EventChannel, init=False)
 
-    def __getitem__(self, reference: type[T]) -> Injectable[T]:
-        cls = self.__get_origin(reference)
+    def __getitem__(self, cls: type[T], /) -> Injectable[T]:
+        origin = self.__get_origin(cls)
 
         try:
-            return self.__data[cls]
+            return self.__data[origin]
         except KeyError as exc:
-            raise NoInjectable(f"No injectable for `{_format_type(cls)}`.") from exc
+            raise NoInjectable(cls) from exc
 
-    def set_multiple(self, references: Iterable[type], injectable: Injectable):
-        references = set(self.__get_origin(reference) for reference in references)
+    def set_multiple(self, classes: Iterable[type], injectable: Injectable):
+        classes = set(self.__get_origin(cls) for cls in classes)
 
-        if references:
-            new_values = (
-                (self.check_if_exists(reference), injectable)
-                for reference in references
-            )
+        if classes:
+            new_values = ((self.check_if_exists(cls), injectable) for cls in classes)
             self.__data.update(new_values)
-            event = ContainerDependenciesUpdated(self, references)
+            event = ContainerDependenciesUpdated(self, classes)
             self.notify(event)
 
         return self
 
-    def check_if_exists(self, reference: type) -> type:
-        if reference in self.__data:
+    def check_if_exists(self, cls: type) -> type:
+        if cls in self.__data:
             raise RuntimeError(
-                "An injectable already exists for the reference "
-                f"class `{_format_type(reference)}`."
+                f"An injectable already exists for the class `{format_type(cls)}`."
             )
 
-        return reference
+        return cls
 
     def add_listener(self, listener: EventListener):
         self.__channel.add_listener(listener)
@@ -257,12 +242,16 @@ class Module(EventListener):
     def __post_init__(self):
         self.__container.add_listener(self)
 
-    def __getitem__(self, reference: type[T], /) -> Injectable[T]:
-        return ChainMap(*self.__modules, self.__container)[reference]
+    def __getitem__(self, cls: type[T], /) -> Injectable[T]:
+        for getter in *self.__modules, self.__container:
+            with suppress(KeyError):
+                return getter[cls]
+
+        raise NoInjectable(cls)
 
     def __setitem__(self, on: type | Iterable[type], injectable: Injectable, /):
-        references = on if isinstance(on, Iterable) else (on,)
-        self.__container.set_multiple(references, injectable)
+        classes = on if isinstance(on, Iterable) else (on,)
+        self.__container.set_multiple(classes, injectable)
 
     def __str__(self) -> str:
         return self.name or object.__str__(self)
@@ -299,19 +288,19 @@ class Module(EventListener):
 
         return InjectableDecorator(self, SingletonInjectable)
 
-    def get_instance(self, reference: type[T]) -> T | None:
+    def get_instance(self, cls: type[T]) -> T | None:
         """
         Function used to retrieve an instance associated with the type passed in
         parameter or return `None`.
         """
 
         try:
-            injectable = self[reference]
+            injectable = self[cls]
         except KeyError:
             return None
 
         instance = injectable.get_instance()
-        return cast(reference, instance)
+        return cast(cls, instance)
 
     def use(
         self,
@@ -390,14 +379,7 @@ class Module(EventListener):
         return self
 
     def on_event(self, event: Event, /):
-        self_event = ModuleEventProxy(self, event)
-
-        if self_event.is_circular:
-            raise ModuleCircularUseError(
-                "Circular dependency between two modules: "
-                f"`{self_event.previous_module}` and `{self}`."
-            )
-
+        self_event = ModuleEventProxy(self, event).check_recursion()
         self.notify(self_event)
 
     def notify(self, event: Event):
@@ -433,7 +415,7 @@ class Dependencies:
             yield name, injectable.get_instance()
 
     @property
-    def arguments(self) -> Mapping[str, Any]:
+    def arguments(self) -> dict[str, Any]:
         return dict(self)
 
     @classmethod
@@ -542,10 +524,10 @@ class InjectDecorator:
 @dataclass(repr=False, frozen=True, slots=True)
 class InjectableDecorator:
     __module: Module
-    __class: type[_BaseInjectable]
+    __injectable_type: type[BaseInjectable]
 
     def __repr__(self) -> str:
-        return f"<{self.__class.__qualname__} decorator>"
+        return f"<{self.__injectable_type.__qualname__} decorator>"
 
     def __call__(
         self,
@@ -559,9 +541,9 @@ class InjectableDecorator:
                 wp = self.__module.inject(wp)
 
             @lambda fn: fn()
-            def references():
-                if reference := self.__get_reference(wp):
-                    yield reference
+            def classes():
+                if cls := self.__get_target_class(wp):
+                    yield cls
 
                 if on is None:
                     return
@@ -570,17 +552,17 @@ class InjectableDecorator:
                 else:
                     yield on
 
-            self.__module[references] = self.__class(wp)
+            self.__module[classes] = self.__injectable_type(wp)
             return wp
 
         return decorator(wrapped) if wrapped else decorator
 
     @staticmethod
-    def __get_reference(wrapped: Callable[..., Any], /) -> type | None:
+    def __get_target_class(wrapped: Callable[..., Any], /) -> type | None:
         if isinstance(wrapped, type):
             return wrapped
 
-        elif callable(wrapped):
+        if callable(wrapped):
             return_type = get_annotations(wrapped, eval_str=True).get("return")
 
             if isinstance(return_type, type):
